(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define([], factory);
  } else if (typeof exports !== "undefined") {
    factory();
  } else {
    var mod = {
      exports: {}
    };
    factory();
    global.webpackProduction = mod.exports;
  }
})(this, function () {
  "use strict";

  var path = require('path');

  var TerserPlugin = require('terser-webpack-plugin');

  var _require = require('webpack-stats-plugin'),
      StatsWriterPlugin = _require.StatsWriterPlugin;

  var _require2 = require('webpack-bundle-analyzer'),
      BundleAnalyzerPlugin = _require2.BundleAnalyzerPlugin;

  module.exports = function (env, argv) {
    var clientProduction = {
      devtool: 'source-map',
      entry: {
        bundle: path.join(process.cwd(), '/.app/production-renderer.js')
      },
      output: {
        path: path.join(process.cwd(), '/public/'),
        publicPath: '/',
        filename: '[name]-[chunkhash].js'
      },
      mode: 'production',
      optimization: {
        runtimeChunk: {
          name: 'bootstrap'
        },
        splitChunks: {
          cacheGroups: {
            vendors: {
              test: /[\\/]node_modules[\\/]/,
              priority: -10
            }
          }
        },
        minimizer: [new TerserPlugin({
          parallel: true,
          cache: true,
          sourceMap: false,
          terserOptions: {
            compress: {
              ecma: 5,
              warnings: false,
              comparisons: false,
              inline: 2
            },
            mangle: {
              safari10: true
            },
            output: {
              ecma: 5,
              comments: false,
              ascii_only: true
            }
          }
        })]
      },
      plugins: [new StatsWriterPlugin()]
    };

    if (env === 'ANALYZE') {
      clientProduction.plugins.push(new BundleAnalyzerPlugin());
    }

    return clientProduction;
  };
});