(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define([], factory);
  } else if (typeof exports !== "undefined") {
    factory();
  } else {
    var mod = {
      exports: {}
    };
    factory();
    global.webpackShared = mod.exports;
  }
})(this, function () {
  "use strict";

  /*
    This webpack configuration is shared between client-webpack, html-generator and routes-generator
   */
  var path = require('path');

  var merge = require('webpack-merge');

  var fs = require('fs');

  var webpack = require('webpack');

  var FriendlyErrors = require('friendly-errors-webpack-plugin');

  var babelOptions = require('./babel-loader.options');

  module.exports = function webpackShared(env, argv) {
    var userConfig = {};
    var processRoot = process.cwd();
    var configPath = path.join(processRoot, 'chido.config.js');

    if (fs.existsSync(configPath)) {
      var _require = require(configPath),
          webpackConfig = _require.webpackConfig;

      if (typeof webpackConfig === 'function') {
        userConfig = webpackConfig(env, argv);
      }
    }

    var config = {
      context: process.cwd(),
      module: {
        rules: [{
          test: /\.jsx?$/,
          loader: 'babel-loader',
          exclude: /node_modules/,
          options: babelOptions
        }] // No .css here please

      },
      resolve: {
        extensions: ['.js', '.css', '.scss', '.sass', '.jsx'],
        alias: {
          'react-dom': '@hot-loader/react-dom'
        }
      },
      plugins: [new webpack.DefinePlugin({
        STATIC: JSON.stringify(env === 'static')
      }), new FriendlyErrors()]
    };
    return merge.smart(config, userConfig);
  };
});