(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define([], factory);
  } else if (typeof exports !== "undefined") {
    factory();
  } else {
    var mod = {
      exports: {}
    };
    factory();
    global.htmlWebpack = mod.exports;
  }
})(this, function () {
  "use strict";

  // Only one entry point
  var path = require('path');

  var StaticSiteGeneratorPlugin = require('static-site-generator-webpack-plugin');

  var webpack = require('webpack');

  var fs = require('fs');

  var webpackMerge = require('webpack-merge');

  var chidoShared = require('./html.shared');
  /* eslint-disable-next-line import/no-unresolved */


  var sharedConfig = require('../client-webpack/webpack.shared');

  module.exports = function ChidoWebpack(env, argv) {
    var rawRoutes = fs.readFileSync(path.join(process.cwd(), '/.app/routes/routes.json'));
    var routes = JSON.parse(rawRoutes.toString());
    var paths = Object.keys(routes);
    var config = {
      mode: 'production',
      entry: path.join(process.cwd(), '/.app/build-static-html.js'),
      output: {
        filename: '.chido__hidden.js',
        path: path.join(process.cwd(), '/public'),
        libraryTarget: 'umd',
        globalObject: 'this'
      },
      plugins: [new StaticSiteGeneratorPlugin({
        paths: paths,
        locals: {
          routes: routes,
          template: fs.readFileSync(path.join(__dirname, '../default-html/index.html'))
        }
      }), new webpack.optimize.LimitChunkCountPlugin({
        maxChunks: 1
      }), new webpack.DefinePlugin({
        IS_STATIC: true,
        'process.env': {
          NODE_ENV: JSON.stringify('production')
        }
      })]
    };
    var shared = sharedConfig(env, argv);
    shared.module.rules = shared.module.rules.filter(function (_ref) {
      var test = _ref.test;
      return String(test) !== String(/\.css$/);
    });
    return webpackMerge.smart(shared, config, chidoShared);
  };
});